#pragma once

// C++ system level
#include <functional>

// ESP32 specific
#include "esp_log.h"
#include "FreeRTOS.h"
#include "freertos/semphr.h"

// project specific
// #include <SpiderLib/Util.hpp>


namespace SpiderLib {

  class ManagedMutex {

    public:
      ManagedMutex() {

        semphr = xSemaphoreCreateBinary();

        if (semphr == NULL) {
          ESP_LOGE("ManagedMutex", "Memory allocation FAILED");
          abort();
        }

        xSemaphoreGive(semphr);
      }

      void run(std::function<void()> body) {

        xSemaphoreTake(semphr, portMAX_DELAY);
        body();
        xSemaphoreGive(semphr);
      }

      template<typename T>
      T runReturn(std::function<T()> body) {

        xSemaphoreTake(semphr, portMAX_DELAY);
        T result = body();
        xSemaphoreGive(semphr);

        return result;
      }

    private:

      SemaphoreHandle_t semphr;

  };

}
